@@ -48,7 +48,7 @@
 #include <vector>
 #include <map>
 
-#include "hsakmt.h"
+#include "hsakmt/hsakmt.h"
 
 #include "core/inc/runtime.h"
 #include "core/inc/agent.h"
@@ -231,6 +231,17 @@ class GpuAgent : public GpuAgentInt {
                        std::vector<core::Signal*>& dep_signals,
                        core::Signal& out_signal) override;
 
+  // @brief Override from core::Agent.
+  hsa_status_t DmaCopyOnEngine(void* dst, core::Agent& dst_agent, const void* src,
+                       core::Agent& src_agent, size_t size,
+                       std::vector<core::Signal*>& dep_signals,
+                       core::Signal& out_signal, int engine_offset,
+                       bool force_copy_on_sdma) override;
+
+  // @brief Override from core::Agent.
+  hsa_status_t DmaCopyStatus(core::Agent& dst_agent, core::Agent& src_agent,
+                             uint32_t *engine_ids_mask) override;
+
   // @brief Override from core::Agent.
   hsa_status_t DmaCopyRect(const hsa_pitched_ptr_t* dst, const hsa_dim3_t* dst_offset,
                            const hsa_pitched_ptr_t* src, const hsa_dim3_t* src_offset,
@@ -283,6 +294,9 @@ class GpuAgent : public GpuAgentInt {
   // @brief Returns Hive ID
   __forceinline uint64_t HiveId() const override { return  properties_.HiveID; }
 
+  // @brief Returns KFD's GPU id which is a hash used internally.
+  __forceinline uint64_t KfdGpuID() const { return properties_.KFDGpuID; }
+
   // @brief Returns node property.
   __forceinline const HsaNodeProperties& properties() const {
     return properties_;
@@ -326,6 +340,11 @@ class GpuAgent : public GpuAgentInt {
   // GPU devices.
   __forceinline uint32_t enumeration_index() const { return enum_index_; }
 
+  // @brief returns true if agent uses MES scheduler
+  __forceinline const bool isMES() const { return (isa_->GetMajorVersion() >= 11) ? true : false; };
+
+  void ReserveScratch();
+
   void Trim() override;
 
   const std::function<void*(size_t size, size_t align, core::MemoryRegion::AllocateFlags flags)>&
@@ -447,6 +466,9 @@ class GpuAgent : public GpuAgentInt {
 
   double historical_clock_ratio_;
 
+  // @brief s_memrealtime nominal clock frequency
+  uint64_t wallclock_frequency_;
+
   // @brief Array of GPU cache property.
   std::vector<HsaCacheProperties> cache_props_;
 
@@ -528,6 +550,10 @@ class GpuAgent : public GpuAgentInt {
   // Bind the Blit object that will drive the copy operation
   lazy_ptr<core::Blit>& GetBlitObject(const core::Agent& dst_agent, const core::Agent& src_agent,
                                       const size_t size);
+
+  // Bind the Blit object that will drive the copy operation by engine ID
+  lazy_ptr<core::Blit>& GetBlitObject(uint32_t engine_id);
+
   // @brief Alternative aperture base address. Only on KV.
   uintptr_t ape1_base_;
 
@@ -541,6 +567,15 @@ class GpuAgent : public GpuAgentInt {
     KernelMutex lock_;
   } gws_queue_;
 
+  // Sets and Tracks pending SDMA status check or request counts
+  void SetCopyRequestRefCount(bool set);
+  void SetCopyStatusCheckRefCount(bool set);
+  int pending_copy_req_ref_;
+  int pending_copy_stat_check_ref_;
+
+  // Tracks what SDMA blits have been used since initialization.
+  uint32_t sdma_blit_used_mask_;
+
   ScratchCache scratch_cache_;
 
   // System memory allocator in the nearest NUMA node.
@@ -550,6 +585,9 @@ class GpuAgent : public GpuAgentInt {
   std::function<void(void*)> system_deallocator_;
 
   DISALLOW_COPY_AND_ASSIGN(GpuAgent);
+
+  // Check if SDMA engine by ID is free
+  bool DmaEngineIsFree(uint32_t engine_id);
 };
 
 }  // namespace amd
