@@ -50,6 +50,7 @@
 #include <memory>
 #include <tuple>
 #include <utility>
+#include <thread>
 
 #include "core/inc/hsa_ext_interface.h"
 #include "core/inc/hsa_internal.h"
@@ -60,6 +61,7 @@
 #include "core/inc/memory_region.h"
 #include "core/inc/signal.h"
 #include "core/inc/interrupt_signal.h"
+#include "core/inc/svm_profiler.h"
 #include "core/util/flag.h"
 #include "core/util/locks.h"
 #include "core/util/os.h"
@@ -68,6 +70,12 @@
 #include "core/inc/amd_loader_context.hpp"
 #include "core/inc/amd_hsa_code.hpp"
 
+#if defined(__clang__)
+#if __has_feature(address_sanitizer)
+#define SANITIZER_AMDGPU 1
+#endif
+#endif
+
 //---------------------------------------------------------------------------//
 //    Constants                                                              //
 //---------------------------------------------------------------------------//
@@ -84,6 +92,7 @@ namespace AMD {
 
 namespace core {
 extern bool g_use_interrupt_wait;
+extern bool g_use_mwaitx;
 
 /// @brief  Runtime class provides the following functions:
 /// - open and close connection to kernel driver.
@@ -109,6 +118,7 @@ class Runtime {
   struct KfdVersion_t {
     HsaVersionInfo version;
     bool supports_exception_debugging;
+    bool supports_event_age;
   };
 
   /// @brief Open connection to kernel driver and increment reference count.
@@ -132,7 +142,7 @@ class Runtime {
 
   /// @brief Insert agent into agent list ::agents_.
   /// @param [in] agent Pointer to the agent object.
-  void RegisterAgent(Agent* agent);
+  void RegisterAgent(Agent* agent, bool Enabled);
 
   /// @brief Delete all agent objects from ::agents_.
   void DestroyAgents();
@@ -221,10 +231,38 @@ class Runtime {
   ///
   /// @retval ::HSA_STATUS_SUCCESS if copy command has been submitted
   /// successfully to the agent DMA queue.
-  hsa_status_t CopyMemory(void* dst, core::Agent& dst_agent, const void* src,
-                          core::Agent& src_agent, size_t size,
-                          std::vector<core::Signal*>& dep_signals,
-                          core::Signal& completion_signal);
+  hsa_status_t CopyMemory(void* dst, core::Agent* dst_agent, const void* src,
+                          core::Agent* src_agent, size_t size,
+                          std::vector<core::Signal*>& dep_signals, core::Signal& completion_signal);
+
+  /// @brief Non-blocking memory copy from src to dst on engine_id.
+  ///
+  /// @details All semantics and params are dentical to CopyMemory
+  ///  with the exception of engine_id.
+  ///
+  /// @param [in] engine_id Target engine to copy on.
+  ///
+  /// @param [in] force_copy_on_sdma By default, a blit kernel copy is used
+  /// when dst_agent == src_agent.  Setting this to true will force the copy
+  /// over SDMA1.
+  ///
+  /// @retval ::HSA_STATUS_SUCCESS if copy command has been submitted
+  /// successfully to the agent DMA queue.
+  hsa_status_t CopyMemoryOnEngine(void* dst, core::Agent* dst_agent, const void* src,
+                          core::Agent* src_agent, size_t size,
+                          std::vector<core::Signal*>& dep_signals, core::Signal& completion_signal,
+                          hsa_amd_sdma_engine_id_t  engine_id, bool force_copy_on_sdma);
+
+  /// @brief Return SDMA availability status for copy direction
+  ///
+  /// @param [in] dst_agent Destination agent.
+  /// @param [in] src_agent Source agent.
+  /// @param [out] engine_ids_mask Mask of engine_ids.
+  ///
+  /// @retval HSA_STATUS_SUCCESS DMA engines are available
+  /// @retval HSA_STATUS_ERROR_OUT_OF_RESOURCES DMA engines are not available
+  hsa_status_t CopyMemoryStatus(core::Agent* dst_agent, core::Agent* src_agent,
+                                uint32_t *engine_ids_mask);
 
   /// @brief Fill the first @p count of uint32_t in ptr with value.
   ///
@@ -282,6 +320,7 @@ class Runtime {
   struct PtrInfoBlockData {
     void* base;
     size_t length;
+    core::Agent* agentOwner;
   };
 
   hsa_status_t PtrInfo(const void* ptr, hsa_amd_pointer_info_t* info, void* (*alloc)(size_t),
@@ -306,12 +345,20 @@ class Runtime {
   hsa_status_t SvmPrefetch(void* ptr, size_t size, hsa_agent_t agent, uint32_t num_dep_signals,
                            const hsa_signal_t* dep_signals, hsa_signal_t completion_signal);
 
+  hsa_status_t DmaBufExport(const void* ptr, size_t size, int* dmabuf, uint64_t* offset);
+
+  hsa_status_t DmaBufClose(int dmabuf);
+
   const std::vector<Agent*>& cpu_agents() { return cpu_agents_; }
 
   const std::vector<Agent*>& gpu_agents() { return gpu_agents_; }
 
+  const std::vector<Agent*>& disabled_gpu_agents() { return disabled_gpu_agents_; }
+
   const std::vector<uint32_t>& gpu_ids() { return gpu_ids_; }
 
+  Agent* agent_by_gpuid(uint32_t gpuid) { return agents_by_gpuid_[gpuid]; }
+
   Agent* region_gpu() { return region_gpu_; }
 
   const std::vector<const MemoryRegion*>& system_regions_fine() const {
@@ -355,7 +402,12 @@ class Runtime {
 
   uint64_t sys_clock_freq() const { return sys_clock_freq_; }
 
-  void KfdVersion(const HsaVersionInfo& version) { kfd_version.version = version; }
+  void KfdVersion(const HsaVersionInfo& version) {
+    kfd_version.version = version;
+    if (version.KernelInterfaceMajorVersion == 1 &&
+      version.KernelInterfaceMinorVersion >= 14)
+      kfd_version.supports_event_age = true;
+  }
 
   void KfdVersion(bool exception_debugging) {
     kfd_version.supports_exception_debugging = exception_debugging;
@@ -367,9 +419,19 @@ class Runtime {
   static void AsyncEventsLoop(void*);
 
   struct AllocationRegion {
-    AllocationRegion() : region(NULL), size(0), user_ptr(nullptr) {}
-    AllocationRegion(const MemoryRegion* region_arg, size_t size_arg)
-        : region(region_arg), size(size_arg), user_ptr(nullptr) {}
+    AllocationRegion()
+        : region(NULL),
+          size(0),
+          size_requested(0),
+          alloc_flags(core::MemoryRegion::AllocateNoFlags),
+          user_ptr(nullptr) {}
+    AllocationRegion(const MemoryRegion* region_arg, size_t size_arg, size_t size_requested,
+                     MemoryRegion::AllocateFlags alloc_flags)
+        : region(region_arg),
+          size(size_arg),
+          size_requested(size_requested),
+          alloc_flags(alloc_flags),
+          user_ptr(nullptr) {}
 
     struct notifier_t {
       void* ptr;
@@ -378,7 +440,9 @@ class Runtime {
     };
 
     const MemoryRegion* region;
-    size_t size;
+    size_t size;           /* actual size = align_up(size_requested, granularity) */
+    size_t size_requested; /* size requested by user */
+    MemoryRegion::AllocateFlags alloc_flags;
     void* user_ptr;
     std::unique_ptr<std::vector<notifier_t>> notifiers;
   };
@@ -505,9 +569,15 @@ class Runtime {
   // Agent list containing all compatible GPU agents in the platform.
   std::vector<Agent*> gpu_agents_;
 
+  // Agent list containing incompletely initialized GPU agents not to be used by the process.
+  std::vector<Agent*> disabled_gpu_agents_;
+
   // Agent map containing all agents indexed by their KFD node IDs.
   std::map<uint32_t, std::vector<Agent*> > agents_by_node_;
 
+  // Agent map containing all agents indexed by their KFD gpuid.
+  std::map<uint32_t, Agent*> agents_by_gpuid_;
+
   // Agent list containing all compatible gpu agent ids in the platform.
   std::vector<uint32_t> gpu_ids_;
 
@@ -590,6 +660,8 @@ class Runtime {
   // Kfd version
   KfdVersion_t kfd_version;
 
+  std::unique_ptr<AMD::SvmProfileControl> svm_profile_;
+
   // Frees runtime memory when the runtime library is unloaded if safe to do so.
   // Failure to release the runtime indicates an incorrect application but is
   // common (example: calls library routines at process exit).
