@@ -80,7 +80,7 @@ int AqlQueue::rtti_id_ = 0;
 
 AqlQueue::AqlQueue(GpuAgent* agent, size_t req_size_pkts, HSAuint32 node_id, ScratchInfo& scratch,
                    core::HsaEventCallback callback, void* err_data, bool is_kv)
-    : Queue(),
+    : Queue(agent->isMES() ? MemoryRegion::AllocateNonPaged : 0),
       LocalSignal(0, false),
       DoorbellSignal(signal()),
       ring_buf_(nullptr),
@@ -213,6 +213,11 @@ AqlQueue::AqlQueue(GpuAgent* agent, size_t req_size_pkts, HSAuint32 node_id, Scr
     assert(amd_queue_.private_segment_aperture_base_hi != 0 && "No private region found.");
   }
 
+  if (agent_->isa()->GetMajorVersion() >= 11)
+    queue_scratch_.mem_alignment_size = 256;
+  else
+    queue_scratch_.mem_alignment_size = 1024;
+
   MAKE_NAMED_SCOPE_GUARD(EventGuard, [&]() {
     ScopedAcquire<KernelMutex> _lock(&queue_lock_);
     queue_count_--;
@@ -280,6 +285,9 @@ AqlQueue::AqlQueue(GpuAgent* agent, size_t req_size_pkts, HSAuint32 node_id, Scr
   queue_id_ = queue_rsrc.QueueId;
   MAKE_NAMED_SCOPE_GUARD(QueueGuard, [&]() { hsaKmtDestroyQueue(queue_id_); });
 
+  // On the first queue creation, reserve some scratch memory on this agent.
+  agent_->ReserveScratch();
+
   // Initialize scratch memory related entities
   queue_scratch_.queue_retry = amd_queue_.queue_inactive_signal;
   InitScratchSRD();
@@ -839,8 +847,10 @@ bool AqlQueue::DynamicScratchHandler(hsa_signal_value_t error_code, void* arg) {
 
       scratch.size_per_thread = scratch_request;
       scratch.lanes_per_wave = (error_code & 0x400) ? 32 : 64;
-      // Align whole waves to 1KB.
-      scratch.size_per_thread = AlignUp(scratch.size_per_thread, 1024 / scratch.lanes_per_wave);
+
+      scratch.size_per_thread =
+          AlignUp(scratch.size_per_thread, scratch.mem_alignment_size / scratch.lanes_per_wave);
+
       scratch.size = scratch.size_per_thread * MaxScratchSlots * scratch.lanes_per_wave;
 
       // Smaller dispatches may not need to reach full device occupancy.
@@ -934,7 +944,7 @@ bool AqlQueue::DynamicScratchHandler(hsa_signal_value_t error_code, void* arg) {
         errorCode = HSA_STATUS_ERROR_INVALID_ARGUMENT;
 
       } else if ((error_code & 128) == 128) {  // Out of VGPRs
-        errorCode = HSA_STATUS_ERROR_INVALID_ISA;
+        errorCode = hsa_status_t(HSA_STATUS_ERROR_OUT_OF_REGISTERS);
 
       } else if ((error_code & 0x20000000) == 0x20000000) {  // Memory violation (>48-bit)
         errorCode = hsa_status_t(HSA_STATUS_ERROR_MEMORY_APERTURE_VIOLATION);
@@ -1116,6 +1126,15 @@ hsa_status_t AqlQueue::SetCUMasking(uint32_t num_cu_mask_count, const uint32_t*
   // Apply mask if non-default or not queue initialization.
   ScopedAcquire<KernelMutex> lock(&mask_lock_);
   if ((!cu_mask_.empty()) || (num_cu_mask_count != 0) || (!global_mask.empty())) {
+
+    // Devices with WGPs must conform to even-indexed contiguous pairwise CU enablement.
+    if (agent_->isa()->GetMajorVersion() >= 10) {
+      for (int i = 0; i < mask.size() * 32; i += 2) {
+        uint32_t cu_pair = (mask[i / 32] >> (i % 32)) & 0x3;
+        if (cu_pair && cu_pair != 0x3) return HSA_STATUS_ERROR_INVALID_ARGUMENT;
+      }
+    }
+
     HSAKMT_STATUS ret =
         hsaKmtSetQueueCUMask(queue_id_, mask.size() * 32, reinterpret_cast<HSAuint32*>(&mask[0]));
     if (ret != HSAKMT_STATUS_SUCCESS) return HSA_STATUS_ERROR;
@@ -1175,6 +1194,8 @@ void AqlQueue::ExecutePM4(uint32_t* cmd_data, size_t cmd_size_b) {
   // To respect multi-producer semantics, first buffer commands for the queue slot.
   constexpr uint32_t slot_size_dw = uint32_t(slot_size_b / sizeof(uint32_t));
   uint32_t slot_data[slot_size_dw];
+  hsa_signal_t signal = {0};
+  hsa_status_t err;
 
   if (agent_->isa()->GetMajorVersion() <= 8) {
     // Construct a set of PM4 to fit inside the AQL packet slot.
@@ -1227,6 +1248,9 @@ void AqlQueue::ExecutePM4(uint32_t* cmd_data, size_t cmd_size_b) {
 
     constexpr uint32_t AMD_AQL_FORMAT_PM4_IB = 0x1;
 
+    err = hsa_signal_create(1, 0, NULL, &signal);
+    assert(err == HSA_STATUS_SUCCESS);
+
     amd_aql_pm4_ib aql_pm4_ib{};
     aql_pm4_ib.header = HSA_PACKET_TYPE_VENDOR_SPECIFIC << HSA_PACKET_HEADER_TYPE;
     aql_pm4_ib.ven_hdr = AMD_AQL_FORMAT_PM4_IB;
@@ -1235,6 +1259,7 @@ void AqlQueue::ExecutePM4(uint32_t* cmd_data, size_t cmd_size_b) {
     aql_pm4_ib.ib_jump_cmd[2] = ib_jump_cmd[2];
     aql_pm4_ib.ib_jump_cmd[3] = ib_jump_cmd[3];
     aql_pm4_ib.dw_cnt_remain = 0xA;
+    aql_pm4_ib.completion_signal = signal;
 
     memcpy(slot_data, &aql_pm4_ib, sizeof(aql_pm4_ib));
   } else {
@@ -1252,85 +1277,226 @@ void AqlQueue::ExecutePM4(uint32_t* cmd_data, size_t cmd_size_b) {
   doorbell->StoreRelease(write_idx);
 
   // Wait for the packet to be consumed.
-  // Should be switched to a signal wait when aql_pm4_ib can be used on all
-  // supported platforms.
-  while (queue->LoadReadIndexRelaxed() <= write_idx) {
-    os::YieldThread();
+  if (agent_->isa()->GetMajorVersion() <= 8) {
+    while (queue->LoadReadIndexRelaxed() <= write_idx)
+      os::YieldThread();
+  } else {
+    hsa_signal_value_t ret;
+    ret = hsa_signal_wait_scacquire(signal, HSA_SIGNAL_CONDITION_LT, 1,
+                                    (uint64_t)-1, HSA_WAIT_STATE_ACTIVE);
+    err = hsa_signal_destroy(signal);
+    assert(ret == 0 && err == HSA_STATUS_SUCCESS);
   }
 }
 
-// @brief Define the Scratch Buffer Descriptor and related parameters
-// that enable kernel access scratch memory
-void AqlQueue::InitScratchSRD() {
-
-  // Populate scratch resource descriptor
+void AqlQueue::FillBufRsrcWord0() {
   SQ_BUF_RSRC_WORD0 srd0;
-  SQ_BUF_RSRC_WORD1 srd1;
-  SQ_BUF_RSRC_WORD2 srd2;
-  uint32_t srd3_u32;
-
-  uint32_t scratch_base_hi = 0;
   uintptr_t scratch_base = uintptr_t(queue_scratch_.queue_base);
-  #ifdef HSA_LARGE_MODEL
+
+  srd0.bits.BASE_ADDRESS = uint32_t(scratch_base);
+  amd_queue_.scratch_resource_descriptor[0] = srd0.u32All;
+}
+
+void AqlQueue::FillBufRsrcWord1() {
+  SQ_BUF_RSRC_WORD1 srd1;
+  uint32_t scratch_base_hi = 0;
+
+#ifdef HSA_LARGE_MODEL
+  uintptr_t scratch_base = uintptr_t(queue_scratch_.queue_base);
   scratch_base_hi = uint32_t(scratch_base >> 32);
   #endif
-  srd0.bits.BASE_ADDRESS = uint32_t(scratch_base);
 
   srd1.bits.BASE_ADDRESS_HI = scratch_base_hi;
   srd1.bits.STRIDE = 0;
   srd1.bits.CACHE_SWIZZLE = 0;
   srd1.bits.SWIZZLE_ENABLE = 1;
 
-  srd2.bits.NUM_RECORDS = uint32_t(queue_scratch_.size);
+  amd_queue_.scratch_resource_descriptor[1] = srd1.u32All;
+}
 
-  if (agent_->isa()->GetMajorVersion() < 10) {
-    SQ_BUF_RSRC_WORD3 srd3;
+void AqlQueue::FillBufRsrcWord1_Gfx11() {
+  SQ_BUF_RSRC_WORD1_GFX11 srd1;
+  uint32_t scratch_base_hi = 0;
 
-    srd3.bits.DST_SEL_X = SQ_SEL_X;
-    srd3.bits.DST_SEL_Y = SQ_SEL_Y;
-    srd3.bits.DST_SEL_Z = SQ_SEL_Z;
-    srd3.bits.DST_SEL_W = SQ_SEL_W;
-    srd3.bits.NUM_FORMAT = BUF_NUM_FORMAT_UINT;
-    srd3.bits.DATA_FORMAT = BUF_DATA_FORMAT_32;
-    srd3.bits.ELEMENT_SIZE = 1;  // 4
-    srd3.bits.INDEX_STRIDE = 3;  // 64
-    srd3.bits.ADD_TID_ENABLE = 1;
-    srd3.bits.ATC__CI__VI = (agent_->profile() == HSA_PROFILE_FULL);
-    srd3.bits.HASH_ENABLE = 0;
-    srd3.bits.HEAP = 0;
-    srd3.bits.MTYPE__CI__VI = 0;
-    srd3.bits.TYPE = SQ_RSRC_BUF;
+#ifdef HSA_LARGE_MODEL
+  uintptr_t scratch_base = uintptr_t(queue_scratch_.queue_base);
+  scratch_base_hi = uint32_t(scratch_base >> 32);
+#endif
 
-    srd3_u32 = srd3.u32All;
-  } else {
-    SQ_BUF_RSRC_WORD3_GFX10 srd3;
+  srd1.bits.BASE_ADDRESS_HI = scratch_base_hi;
+  srd1.bits.STRIDE = 0;
+  srd1.bits.SWIZZLE_ENABLE = 1;
 
-    srd3.bits.DST_SEL_X = SQ_SEL_X;
-    srd3.bits.DST_SEL_Y = SQ_SEL_Y;
-    srd3.bits.DST_SEL_Z = SQ_SEL_Z;
-    srd3.bits.DST_SEL_W = SQ_SEL_W;
-    srd3.bits.FORMAT = BUF_FORMAT_32_UINT;
-    srd3.bits.RESERVED1 = 0;
-    srd3.bits.INDEX_STRIDE = 0;  // filled in by CP
-    srd3.bits.ADD_TID_ENABLE = 1;
-    srd3.bits.RESOURCE_LEVEL = 1;
-    srd3.bits.RESERVED2 = 0;
-    srd3.bits.OOB_SELECT = 2;  // no bounds check in swizzle mode
-    srd3.bits.TYPE = SQ_RSRC_BUF;
+  amd_queue_.scratch_resource_descriptor[1] = srd1.u32All;
+}
 
-    srd3_u32 = srd3.u32All;
+void AqlQueue::FillBufRsrcWord2() {
+  SQ_BUF_RSRC_WORD2 srd2;
+  const auto& agent_props = agent_->properties();
+  const uint32_t num_xcc = agent_props.NumXcc;
+
+   // report size per XCC
+  srd2.bits.NUM_RECORDS = uint32_t(queue_scratch_.size / num_xcc);
+
+  amd_queue_.scratch_resource_descriptor[2] = srd2.u32All;
+}
+
+void AqlQueue::FillBufRsrcWord3() {
+  SQ_BUF_RSRC_WORD3 srd3;
+
+  srd3.bits.DST_SEL_X = SQ_SEL_X;
+  srd3.bits.DST_SEL_Y = SQ_SEL_Y;
+  srd3.bits.DST_SEL_Z = SQ_SEL_Z;
+  srd3.bits.DST_SEL_W = SQ_SEL_W;
+  srd3.bits.NUM_FORMAT = BUF_NUM_FORMAT_UINT;
+  srd3.bits.DATA_FORMAT = BUF_DATA_FORMAT_32;
+  srd3.bits.ELEMENT_SIZE = 1;  // 4
+  srd3.bits.INDEX_STRIDE = 3;  // 64
+  srd3.bits.ADD_TID_ENABLE = 1;
+  srd3.bits.ATC__CI__VI = (agent_->profile() == HSA_PROFILE_FULL);
+  srd3.bits.HASH_ENABLE = 0;
+  srd3.bits.HEAP = 0;
+  srd3.bits.MTYPE__CI__VI = 0;
+  srd3.bits.TYPE = SQ_RSRC_BUF;
+
+  amd_queue_.scratch_resource_descriptor[3] = srd3.u32All;
+}
+
+void AqlQueue::FillBufRsrcWord3_Gfx10() {
+  SQ_BUF_RSRC_WORD3_GFX10 srd3;
+
+  srd3.bits.DST_SEL_X = SQ_SEL_X;
+  srd3.bits.DST_SEL_Y = SQ_SEL_Y;
+  srd3.bits.DST_SEL_Z = SQ_SEL_Z;
+  srd3.bits.DST_SEL_W = SQ_SEL_W;
+  srd3.bits.FORMAT = BUF_FORMAT_32_UINT;
+  srd3.bits.RESERVED1 = 0;
+  srd3.bits.INDEX_STRIDE = 0;  // filled in by CP
+  srd3.bits.ADD_TID_ENABLE = 1;
+  srd3.bits.RESOURCE_LEVEL = 1;
+  srd3.bits.RESERVED2 = 0;
+  srd3.bits.OOB_SELECT = 2;  // no bounds check in swizzle mode
+  srd3.bits.TYPE = SQ_RSRC_BUF;
+
+  amd_queue_.scratch_resource_descriptor[3] = srd3.u32All;
+}
+
+void AqlQueue::FillBufRsrcWord3_Gfx11() {
+  SQ_BUF_RSRC_WORD3_GFX11 srd3;
+
+  srd3.bits.DST_SEL_X = SQ_SEL_X;
+  srd3.bits.DST_SEL_Y = SQ_SEL_Y;
+  srd3.bits.DST_SEL_Z = SQ_SEL_Z;
+  srd3.bits.DST_SEL_W = SQ_SEL_W;
+  srd3.bits.FORMAT = BUF_FORMAT_32_UINT;
+  srd3.bits.RESERVED1 = 0;
+  srd3.bits.INDEX_STRIDE = 0;  // filled in by CP
+  srd3.bits.ADD_TID_ENABLE = 1;
+  srd3.bits.RESERVED2 = 0;
+  srd3.bits.OOB_SELECT = 2;  // no bounds check in swizzle mode
+  srd3.bits.TYPE = SQ_RSRC_BUF;
+
+  amd_queue_.scratch_resource_descriptor[3] = srd3.u32All;
+}
+
+// Set concurrent wavefront limits only when scratch is being used.
+void AqlQueue::FillComputeTmpRingSize() {
+  COMPUTE_TMPRING_SIZE tmpring_size = {};
+  if (queue_scratch_.size == 0) {
+    amd_queue_.compute_tmpring_size = tmpring_size.u32All;
+    return;
   }
 
-  // Update Queue's Scratch descriptor's property
-  amd_queue_.scratch_resource_descriptor[0] = srd0.u32All;
-  amd_queue_.scratch_resource_descriptor[1] = srd1.u32All;
-  amd_queue_.scratch_resource_descriptor[2] = srd2.u32All;
-  amd_queue_.scratch_resource_descriptor[3] = srd3_u32;
+  const auto& agent_props = agent_->properties();
+  const uint32_t num_xcc = agent_props.NumXcc;
+
+  // Determine the maximum number of waves device can support
+  uint32_t num_cus = agent_props.NumFComputeCores / agent_props.NumSIMDPerCU;
+  uint32_t max_scratch_waves = num_cus * agent_props.MaxSlotsScratchCU;
+
+  // Scratch is allocated program COMPUTE_TMPRING_SIZE register
+  // Scratch Size per Wave is specified in terms of kilobytes
+  uint32_t wave_scratch = (((queue_scratch_.lanes_per_wave * queue_scratch_.size_per_thread) +
+                            queue_scratch_.mem_alignment_size - 1) /
+                           queue_scratch_.mem_alignment_size);
+  tmpring_size.bits.WAVESIZE = wave_scratch;
+  assert(wave_scratch == tmpring_size.bits.WAVESIZE && "WAVESIZE Overflow.");
+  uint32_t num_waves =
+      (queue_scratch_.size / num_xcc) / (tmpring_size.bits.WAVESIZE * queue_scratch_.mem_alignment_size);
+
+  tmpring_size.bits.WAVES = std::min(num_waves, max_scratch_waves);
+  amd_queue_.compute_tmpring_size = tmpring_size.u32All;
+  assert((tmpring_size.bits.WAVES % (agent_props.NumShaderBanks / num_xcc) == 0) &&
+         "Invalid scratch wave count.  Must be divisible by #SEs.");
+}
+
+// Set concurrent wavefront limits only when scratch is being used.
+void AqlQueue::FillComputeTmpRingSize_Gfx11() {
+  COMPUTE_TMPRING_SIZE_GFX11 tmpring_size = {};
+  if (queue_scratch_.size == 0) {
+    amd_queue_.compute_tmpring_size = tmpring_size.u32All;
+    return;
+  }
+
+  const auto& agent_props = agent_->properties();
+  const uint32_t num_xcc = agent_props.NumXcc;
+
+  // Determine the maximum number of waves device can support
+  uint32_t num_cus = agent_props.NumFComputeCores / (agent_props.NumSIMDPerCU * num_xcc);
+  uint32_t max_scratch_waves = num_cus * agent_props.MaxSlotsScratchCU;
+
+  // Scratch is allocated program COMPUTE_TMPRING_SIZE register
+  // Scratch Size per Wave is specified in terms of kilobytes
+  uint32_t wave_scratch = (((queue_scratch_.lanes_per_wave * queue_scratch_.size_per_thread) +
+                            queue_scratch_.mem_alignment_size - 1) /
+                           queue_scratch_.mem_alignment_size);
+
+  tmpring_size.bits.WAVESIZE = wave_scratch;
+  assert(wave_scratch == tmpring_size.bits.WAVESIZE && "WAVESIZE Overflow.");
+
+  uint32_t num_waves =
+      queue_scratch_.size / (tmpring_size.bits.WAVESIZE * queue_scratch_.mem_alignment_size);
+
+  // For GFX11 we specify number of waves per engine instead of total
+  num_waves /= agent_->properties().NumShaderBanks;
+  tmpring_size.bits.WAVES = std::min(num_waves, max_scratch_waves);
+  amd_queue_.compute_tmpring_size = tmpring_size.u32All;
+}
+
+// @brief Define the Scratch Buffer Descriptor and related parameters
+// that enable kernel access scratch memory
+void AqlQueue::InitScratchSRD() {
+  switch (agent_->isa()->GetMajorVersion()) {
+    case 11:
+      FillBufRsrcWord0();
+      FillBufRsrcWord1_Gfx11();
+      FillBufRsrcWord2();
+      FillBufRsrcWord3_Gfx11();
+      FillComputeTmpRingSize_Gfx11();
+      break;
+    case 10:
+      FillBufRsrcWord0();
+      FillBufRsrcWord1();
+      FillBufRsrcWord2();
+      FillBufRsrcWord3_Gfx10();
+      FillComputeTmpRingSize();
+      break;
+    default:
+      FillBufRsrcWord0();
+      FillBufRsrcWord1();
+      FillBufRsrcWord2();
+      FillBufRsrcWord3();
+      FillComputeTmpRingSize();
+      break;
+  }
 
   // Populate flat scratch parameters in amd_queue_.
-  amd_queue_.scratch_backing_memory_location =
-      queue_scratch_.queue_process_offset;
-  amd_queue_.scratch_backing_memory_byte_size = queue_scratch_.size;
+  amd_queue_.scratch_backing_memory_location = queue_scratch_.queue_process_offset;
+
+  const auto& agent_props = agent_->properties();
+  const uint32_t num_xcc = agent_props.NumXcc;
+  // report size per XCC
+  amd_queue_.scratch_backing_memory_byte_size = queue_scratch_.size / num_xcc;
 
   // For backwards compatibility this field records the per-lane scratch
   // for a 64 lane wavefront. If scratch was allocated for 32 lane waves
@@ -1338,29 +1504,6 @@ void AqlQueue::InitScratchSRD() {
   amd_queue_.scratch_wave64_lane_byte_size =
       uint32_t((queue_scratch_.size_per_thread * queue_scratch_.lanes_per_wave) / 64);
 
-  // Set concurrent wavefront limits only when scratch is being used.
-  COMPUTE_TMPRING_SIZE tmpring_size = {};
-  if (queue_scratch_.size == 0) {
-    amd_queue_.compute_tmpring_size = tmpring_size.u32All;
-    return;
-  }
-
-  // Determine the maximum number of waves device can support
-  const auto& agent_props = agent_->properties();
-  uint32_t num_cus = agent_props.NumFComputeCores / agent_props.NumSIMDPerCU;
-  uint32_t max_scratch_waves = num_cus * agent_props.MaxSlotsScratchCU;
-
-  // Scratch is allocated program COMPUTE_TMPRING_SIZE register
-  // Scratch Size per Wave is specified in terms of kilobytes
-  uint32_t wave_scratch = (((queue_scratch_.lanes_per_wave *
-                               queue_scratch_.size_per_thread) + 1023) / 1024);
-  tmpring_size.bits.WAVESIZE = wave_scratch;
-  assert(wave_scratch == tmpring_size.bits.WAVESIZE && "WAVESIZE Overflow.");
-  uint32_t num_waves = queue_scratch_.size / (tmpring_size.bits.WAVESIZE * 1024);
-  tmpring_size.bits.WAVES = std::min(num_waves, max_scratch_waves);
-  amd_queue_.compute_tmpring_size = tmpring_size.u32All;
-  assert((tmpring_size.bits.WAVES % agent_props.NumShaderBanks == 0) &&
-         "Invalid scratch wave count.  Must be divisible by #SEs.");
   return;
 }
 
