@@ -100,20 +100,28 @@ void MemoryRegion::MakeKfdMemoryUnresident(const void* ptr) {
   hsaKmtUnmapMemoryToGPU(const_cast<void*>(ptr));
 }
 
-MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile, core::Agent* owner,
+MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
+                           bool extended_scope_fine_grain, core::Agent* owner,
                            const HsaMemoryProperties& mem_props)
     : core::MemoryRegion(fine_grain, kernarg, full_profile, owner),
       mem_props_(mem_props),
+      extended_scope_fine_grain_(extended_scope_fine_grain),
       max_single_alloc_size_(0),
       virtual_size_(0),
       fragment_allocator_(BlockAllocator(*this)) {
   virtual_size_ = GetPhysicalSize();
 
+  // extended_scope_fine_grain and fine_grain memory regions are mutually exclusive
+  assert(!(fine_grain && extended_scope_fine_grain));
+
   mem_flag_.Value = 0;
   map_flag_.Value = 0;
-
   static const HSAuint64 kGpuVmSize = (1ULL << 40);
 
+  // Bind the memory region based on whether it is
+  // coarse or fine grain or extended scope fine grain.
+  mem_flag_.ui32.CoarseGrain = (fine_grain || extended_scope_fine_grain) ? 0 : 1;
+
   if (IsLocalMemory()) {
     mem_flag_.ui32.PageSize = HSA_PAGE_SIZE_4KB;
     mem_flag_.ui32.NoSubstitute = 1;
@@ -122,6 +130,20 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile, cor
     mem_flag_.ui32.NonPaged = 1;
 
     virtual_size_ = kGpuVmSize;
+
+    // If memory region is extended scope fine grained
+    // mark the page table entries for this memory region
+    // as MTYPE_UC. Full read and write ordering are guaranteed
+    // to this address.
+    if (extended_scope_fine_grain) {
+      AMD::GpuAgent* agent_ =
+          const_cast<AMD::GpuAgent*>(reinterpret_cast<const AMD::GpuAgent*>(owner));
+      if (agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 0) ||
+          agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 1) ||
+          agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 2))
+        mem_flag_.ui32.Uncached = 1;
+    }
+
   } else if (IsSystem()) {
     mem_flag_.ui32.PageSize = HSA_PAGE_SIZE_4KB;
     mem_flag_.ui32.NoSubstitute = 0;
@@ -134,8 +156,6 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile, cor
         (full_profile) ? os::GetUserModeVirtualMemorySize() : kGpuVmSize;
   }
 
-  // Bind if memory region is coarse or fine grain
-  mem_flag_.ui32.CoarseGrain = (fine_grain) ? 0 : 1;
 
   // Adjust allocatable size per page align
   max_single_alloc_size_ = AlignDown(static_cast<size_t>(GetPhysicalSize()), kPageSize_);
@@ -185,9 +205,12 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
       (alloc_flags & AllocateExecutable ? 1 : 0);
   kmt_alloc_flags.ui32.AQLQueueMemory =
       (alloc_flags & AllocateDoubleMap ? 1 : 0);
-  if (IsSystem() && (alloc_flags & AllocateIPC))
+  if (IsSystem() && (alloc_flags & AllocateNonPaged))
       kmt_alloc_flags.ui32.NonPaged = 1;
 
+  // Allocate pseudo fine grain memory
+  kmt_alloc_flags.ui32.CoarseGrain = (alloc_flags & AllocatePCIeRW ? 0 : kmt_alloc_flags.ui32.CoarseGrain);
+
   // Only allow using the suballocator for ordinary VRAM.
   if (IsLocalMemory()) {
     bool subAllocEnabled = !core::Runtime::runtime_singleton_->flag().disable_fragment_alloc();
@@ -196,6 +219,13 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
     useSubAlloc &= ((alloc_flags & (~AllocateRestrict)) == 0);
     if (useSubAlloc) {
       *address = fragment_allocator_.alloc(size);
+
+      if ((alloc_flags & AllocateAsan) &&
+          hsaKmtReplaceAsanHeaderPage(*address) != HSAKMT_STATUS_SUCCESS) {
+        fragment_allocator_.free(*address);
+        *address = NULL;
+        return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
+      }
       return HSA_STATUS_SUCCESS;
     }
   }
@@ -250,6 +280,12 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
       return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
     }
 
+    if ((alloc_flags & AllocateAsan) &&
+        hsaKmtReplaceAsanHeaderPage(*address) != HSAKMT_STATUS_SUCCESS) {
+      FreeKfdMemory(*address, size);
+      *address = NULL;
+      return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
+    }
     return HSA_STATUS_SUCCESS;
   }
 
@@ -301,8 +337,12 @@ hsa_status_t MemoryRegion::GetInfo(hsa_region_info_t attribute,
         case HSA_HEAPTYPE_SYSTEM:
         case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
         case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE: {
-          uint32_t ret = fine_grain() ? HSA_REGION_GLOBAL_FLAG_FINE_GRAINED
-                                      : HSA_REGION_GLOBAL_FLAG_COARSE_GRAINED;
+          uint32_t ret = 0;
+
+          ret = fine_grain()                ? HSA_REGION_GLOBAL_FLAG_FINE_GRAINED
+              : extended_scope_fine_grain() ? HSA_REGION_GLOBAL_FLAG_EXTENDED_SCOPE_FINE_GRAINED
+                                            : HSA_REGION_GLOBAL_FLAG_COARSE_GRAINED;
+
           if (kernarg()) ret |= HSA_REGION_GLOBAL_FLAG_KERNARG;
           *((uint32_t*)value) = ret;
           break;
@@ -416,6 +456,14 @@ hsa_status_t MemoryRegion::GetPoolInfo(hsa_amd_memory_pool_info_t attribute,
           *((size_t*)value) = 0;
       }
       break;
+    case HSA_AMD_MEMORY_POOL_INFO_LOCATION:
+      if (IsLocalMemory())
+        *((hsa_amd_memory_pool_location_t*)value) = HSA_AMD_MEMORY_POOL_LOCATION_GPU;
+      else if (IsSystem())
+        *((hsa_amd_memory_pool_location_t*)value) = HSA_AMD_MEMORY_POOL_LOCATION_CPU;
+      else
+        return HSA_STATUS_ERROR_INVALID_ARGUMENT;
+      break;
     default:
       return HSA_STATUS_ERROR_INVALID_ARGUMENT;
   }
@@ -447,28 +495,16 @@ hsa_amd_memory_pool_access_t MemoryRegion::GetAccessInfo(
 
   // Determine access type for device local memory which is
   // guaranteed to be HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC
-  // Return disallowed by default if framebuffer is coarse grained
-  // without regard to type of requesting device (CPU / GPU)
-  // Return disallowed by default if framebuffer is fine grained
-  // and requesting device is connected via xGMI link
-  // Return never allowed if framebuffer is fine grained and
-  // requesting device is connected via PCIe link
+
   if (IsLocalMemory()) {
-
     // Return disallowed by default if memory is coarse
-    // grained without regard to link type
-    if  (fine_grain() == false) {
-      return HSA_AMD_MEMORY_POOL_ACCESS_DISALLOWED_BY_DEFAULT;
-    }
-
-    // Determine if pool is pseudo fine-grained due to env flag
-    // Return disallowed by default
-    if (core::Runtime::runtime_singleton_->flag().fine_grain_pcie()) {
+    // grained or extended scope fine grained without regard to link type
+    if (fine_grain() == false) {
       return HSA_AMD_MEMORY_POOL_ACCESS_DISALLOWED_BY_DEFAULT;
     }
 
     // Return disallowed by default if memory is fine
-    // grained and link type is xGMI.
+    // grained and requesting device is connected via xGMI link
     if (agent.HiveId() == owner()->HiveId()) {
       return HSA_AMD_MEMORY_POOL_ACCESS_DISALLOWED_BY_DEFAULT;
     }
@@ -540,10 +576,13 @@ hsa_status_t MemoryRegion::AllowAccess(uint32_t num_agents,
   info.size = sizeof(info);
 
   ScopedAcquire<KernelMutex> lock(&access_lock_);
+
   if (core::Runtime::runtime_singleton_->PtrInfo(const_cast<void*>(ptr), &info, malloc,
                                                  &agent_count, &accessible,
                                                  &blockInfo) == HSA_STATUS_SUCCESS) {
-    if (blockInfo.length != size || info.sizeInBytes != size) {
+    /*  Thunk may return type = HSA_EXT_POINTER_TYPE_UNKNOWN for userptrs */
+    if (info.type != HSA_EXT_POINTER_TYPE_UNKNOWN &&
+        (blockInfo.length != size || info.sizeInBytes != size)) {
       for (int i = 0; i < num_agents; i++) union_agents.push_back(agents[i].handle);
       for (int i = 0; i < agent_count; i++) union_agents.push_back(accessible[i].handle);
       std::sort(union_agents.begin(), union_agents.end());
