@@ -46,8 +46,8 @@
 #include <atomic>
 #include <cstring>
 #include <string>
-#include <thread>
 #include <vector>
+#include <list>
 
 #include "core/common/shared.h"
 #include "core/inc/hsa_ext_interface.h"
@@ -71,6 +71,7 @@ const char rocrbuildid[] __attribute__((used)) = "ROCR BUILD ID: " STRING(ROCR_B
 namespace rocr {
 namespace core {
 bool g_use_interrupt_wait = true;
+bool g_use_mwaitx = true;
 
 Runtime* Runtime::runtime_singleton_ = NULL;
 
@@ -150,7 +151,7 @@ bool Runtime::IsOpen() {
 
 // Register agent information only.  Must not call anything that may use the registered information
 // since those tables are incomplete.
-void Runtime::RegisterAgent(Agent* agent) {
+void Runtime::RegisterAgent(Agent* agent, bool Enabled) {
   // Record the agent in the node-to-agent reverse lookup table.
   agents_by_node_[agent->node_id()].push_back(agent);
 
@@ -158,6 +159,8 @@ void Runtime::RegisterAgent(Agent* agent) {
   if (agent->device_type() == Agent::DeviceType::kAmdCpuDevice) {
     cpu_agents_.push_back(agent);
 
+    agents_by_gpuid_[0] = agent;
+
     // Add cpu regions to the system region list.
     for (const core::MemoryRegion* region : agent->regions()) {
       if (region->fine_grain()) {
@@ -199,12 +202,16 @@ void Runtime::RegisterAgent(Agent* agent) {
       }
     }
   } else if (agent->device_type() == Agent::DeviceType::kAmdGpuDevice) {
-    gpu_agents_.push_back(agent);
+    if (Enabled) {
+      gpu_agents_.push_back(agent);
+      gpu_ids_.push_back(agent->node_id());
+      agents_by_gpuid_[((AMD::GpuAgent*)agent)->KfdGpuID()] = agent;
 
-    gpu_ids_.push_back(agent->node_id());
-
-    // Assign the first discovered gpu agent as region gpu.
-    if (region_gpu_ == NULL) region_gpu_ = agent;
+      // Assign the first discovered gpu agent as region gpu.
+      if (region_gpu_ == NULL) region_gpu_ = agent;
+    } else {
+      disabled_gpu_agents_.push_back(agent);
+    }
   }
 }
 
@@ -214,6 +221,9 @@ void Runtime::DestroyAgents() {
   std::for_each(gpu_agents_.begin(), gpu_agents_.end(), DeleteObject());
   gpu_agents_.clear();
 
+  std::for_each(disabled_gpu_agents_.begin(), disabled_gpu_agents_.end(), DeleteObject());
+  disabled_gpu_agents_.clear();
+
   gpu_ids_.clear();
 
   std::for_each(cpu_agents_.begin(), cpu_agents_.end(), DeleteObject());
@@ -276,11 +286,12 @@ hsa_status_t Runtime::IterateAgent(hsa_status_t (*callback)(hsa_agent_t agent,
 hsa_status_t Runtime::AllocateMemory(const MemoryRegion* region, size_t size,
                                      MemoryRegion::AllocateFlags alloc_flags,
                                      void** address) {
+  size_t size_requested = size;  // region->Allocate(...) may align-up size to granularity
   hsa_status_t status = region->Allocate(size, alloc_flags, address);
   // Track the allocation result so that it could be freed properly.
   if (status == HSA_STATUS_SUCCESS) {
     ScopedAcquire<KernelSharedMutex> lock(&memory_lock_);
-    allocation_map_[*address] = AllocationRegion(region, size);
+    allocation_map_[*address] = AllocationRegion(region, size, size_requested, alloc_flags);
   }
 
   return status;
@@ -294,6 +305,7 @@ hsa_status_t Runtime::FreeMemory(void* ptr) {
   const MemoryRegion* region = nullptr;
   size_t size = 0;
   std::unique_ptr<std::vector<AllocationRegion::notifier_t>> notifiers;
+  MemoryRegion::AllocateFlags alloc_flags = core::MemoryRegion::AllocateNoFlags;
 
   {
     ScopedAcquire<KernelSharedMutex> lock(&memory_lock_);
@@ -306,6 +318,7 @@ hsa_status_t Runtime::FreeMemory(void* ptr) {
     }
     region = it->second.region;
     size = it->second.size;
+    alloc_flags = it->second.alloc_flags;
 
     // Imported fragments can't be released with FreeMemory.
     if (region == nullptr) {
@@ -327,6 +340,9 @@ hsa_status_t Runtime::FreeMemory(void* ptr) {
     }
   }
 
+  if (alloc_flags & core::MemoryRegion::AllocateAsan)
+    assert(hsaKmtReturnAsanHeaderPage(ptr) == HSAKMT_STATUS_SUCCESS);
+
   return region->Free(ptr, size);
 }
 
@@ -449,7 +465,7 @@ hsa_status_t Runtime::CopyMemory(void* dst, const void* src, size_t size) {
 
   /*
   GPU-GPU - functional support, not a performance path.
-  
+
   This goes through system memory because we have to support copying between non-peer GPUs
   and we can't use P2P pointers even if the GPUs are peers.  Because hsa_amd_agents_allow_access
   requires the caller to specify all allowed agents we can't assume that a peer mapped pointer
@@ -462,53 +478,63 @@ hsa_status_t Runtime::CopyMemory(void* dst, const void* src, size_t size) {
   return err;
 }
 
-hsa_status_t Runtime::CopyMemory(void* dst, core::Agent& dst_agent,
-                                 const void* src, core::Agent& src_agent,
-                                 size_t size,
+hsa_status_t Runtime::CopyMemory(void* dst, core::Agent* dst_agent, const void* src,
+                                 core::Agent* src_agent, size_t size,
                                  std::vector<core::Signal*>& dep_signals,
                                  core::Signal& completion_signal) {
-  const bool dst_gpu =
-      (dst_agent.device_type() == core::Agent::DeviceType::kAmdGpuDevice);
-  const bool src_gpu =
-      (src_agent.device_type() == core::Agent::DeviceType::kAmdGpuDevice);
-  if (dst_gpu || src_gpu) {
-    core::Agent* copy_agent = (src_gpu) ? &src_agent : &dst_agent;
-    return copy_agent->DmaCopy(dst, dst_agent, src, src_agent, size, dep_signals,
-                               completion_signal);
+  auto lookupAgent = [this](core::Agent* agent, const void* ptr) {
+    hsa_amd_pointer_info_t info;
+    PtrInfoBlockData block;
+    info.size = sizeof(info);
+    PtrInfo(ptr, &info, nullptr, nullptr, nullptr, &block);
+    // Limit to IPC and GFX types for now.  These are the only types for which the application may
+    // not posess a proper agent handle.
+    if ((info.type != HSA_EXT_POINTER_TYPE_IPC) && (info.type != HSA_EXT_POINTER_TYPE_GRAPHICS)) {
+      return agent;
+    }
+    return block.agentOwner;
+  };
+
+  const bool src_gpu = (src_agent->device_type() == core::Agent::DeviceType::kAmdGpuDevice);
+  core::Agent* copy_agent = (src_gpu) ? src_agent : dst_agent;
+
+  // Lookup owning agent if blit kernel is selected or if flag override is set.
+  if ((dst_agent == src_agent) || flag().discover_copy_agents()) {
+    dst_agent = lookupAgent(dst_agent, dst);
+    src_agent = lookupAgent(src_agent, src);
+  }
+  return copy_agent->DmaCopy(dst, *dst_agent, src, *src_agent, size, dep_signals,
+                             completion_signal);
+}
+
+hsa_status_t Runtime::CopyMemoryOnEngine(void* dst, core::Agent* dst_agent, const void* src,
+                                 core::Agent* src_agent, size_t size,
+                                 std::vector<core::Signal*>& dep_signals,
+                                 core::Signal& completion_signal,
+                                 hsa_amd_sdma_engine_id_t engine_id, bool force_copy_on_sdma) {
+  const bool src_gpu = (src_agent->device_type() == core::Agent::DeviceType::kAmdGpuDevice);
+  core::Agent* copy_agent = (src_gpu) ? src_agent : dst_agent;
+
+  // engine_id is single bitset unique.
+  int engine_offset = ffs(engine_id);
+  if (!engine_id || !!((engine_id >> engine_offset))) {
+    return HSA_STATUS_ERROR_INVALID_ARGUMENT;
   }
 
-  // For cpu to cpu, fire and forget a copy thread.
-  const bool profiling_enabled =
-      (dst_agent.profiling_enabled() || src_agent.profiling_enabled());
-  if (profiling_enabled) completion_signal.async_copy_agent(&dst_agent);
-  std::thread(
-      [](void* dst, const void* src, size_t size,
-         std::vector<core::Signal*> dep_signals,
-         core::Signal* completion_signal, bool profiling_enabled) {
+  return copy_agent->DmaCopyOnEngine(dst, *dst_agent, src, *src_agent, size, dep_signals,
+                             completion_signal, engine_offset, force_copy_on_sdma);
+}
 
-        for (core::Signal* dep : dep_signals) {
-          dep->WaitRelaxed(HSA_SIGNAL_CONDITION_EQ, 0, UINT64_MAX,
-                           HSA_WAIT_STATE_BLOCKED);
-        }
+hsa_status_t Runtime::CopyMemoryStatus(core::Agent* dst_agent, core::Agent* src_agent,
+                                       uint32_t *engine_ids_mask) {
+  const bool src_gpu = (src_agent->device_type() == core::Agent::DeviceType::kAmdGpuDevice);
+  core::Agent* copy_agent = (src_gpu) ? src_agent : dst_agent;
 
-        if (profiling_enabled) {
-          core::Runtime::runtime_singleton_->GetSystemInfo(HSA_SYSTEM_INFO_TIMESTAMP,
-                                                           &completion_signal->signal_.start_ts);
-        }
+  if (dst_agent == src_agent) {
+    return HSA_STATUS_ERROR_INVALID_AGENT;
+  }
 
-        memcpy(dst, src, size);
-
-        if (profiling_enabled) {
-          core::Runtime::runtime_singleton_->GetSystemInfo(HSA_SYSTEM_INFO_TIMESTAMP,
-                                                           &completion_signal->signal_.end_ts);
-        }
-
-        completion_signal->SubRelease(1);
-      },
-      dst, src, size, dep_signals, &completion_signal,
-      profiling_enabled).detach();
-
-  return HSA_STATUS_SUCCESS;
+  return copy_agent->DmaCopyStatus(*dst_agent, *src_agent, engine_ids_mask);
 }
 
 hsa_status_t Runtime::FillMemory(void* ptr, uint32_t value, size_t count) {
@@ -565,6 +591,12 @@ hsa_status_t Runtime::AllowAccess(uint32_t num_agents,
     }
 
     amd_region = reinterpret_cast<const AMD::MemoryRegion*>(it->second.region);
+
+    // Imported IPC handle entries inside allocation_map_ do not have an amd_region because they
+    // were allocated in the other process. Access is already granted during IPCAttach().
+    if (!amd_region)
+      return HSA_STATUS_SUCCESS;
+
     alloc_size = it->second.size;
   }
 
@@ -580,9 +612,7 @@ hsa_status_t Runtime::GetSystemInfo(hsa_system_info_t attribute, void* value) {
       *((uint16_t*)value) = HSA_VERSION_MINOR;
       break;
     case HSA_SYSTEM_INFO_TIMESTAMP: {
-      HsaClockCounters clocks;
-      hsaKmtGetClockCounters(0, &clocks);
-      *((uint64_t*)value) = clocks.SystemClockCounter;
+      *((uint64_t*)value) = os::ReadSystemClock();
       break;
     }
     case HSA_SYSTEM_INFO_TIMESTAMP_FREQUENCY: {
@@ -656,6 +686,22 @@ hsa_status_t Runtime::GetSystemInfo(hsa_system_info_t attribute, void* value) {
       *(bool*)value = ret;
       break;
     }
+    case HSA_AMD_SYSTEM_INFO_MWAITX_ENABLED: {
+      *((bool*)value) = g_use_mwaitx;
+      break;
+    }
+    case HSA_AMD_SYSTEM_INFO_DMABUF_SUPPORTED: {
+      auto kfd_version = core::Runtime::runtime_singleton_->KfdVersion().version;
+
+      // Implemented in KFD in 1.12
+      if (kfd_version.KernelInterfaceMajorVersion > 1 ||
+          (kfd_version.KernelInterfaceMajorVersion == 1 &&
+              kfd_version.KernelInterfaceMinorVersion >= 12))
+        *(reinterpret_cast<bool*>(value)) = true;
+      else
+        *(reinterpret_cast<bool*>(value)) = false;
+      break;
+    }
     default:
       return HSA_STATUS_ERROR_INVALID_ARGUMENT;
   }
@@ -777,9 +823,13 @@ hsa_status_t Runtime::PtrInfo(const void* ptr, hsa_amd_pointer_info_t* info, voi
   // check output struct has an initialized size.
   if (info->size == 0) return HSA_STATUS_ERROR_INVALID_ARGUMENT;
 
+  retInfo.size = Min(size_t(info->size), sizeof(hsa_amd_pointer_info_t));
+
   bool returnListData =
       ((alloc != nullptr) && (num_agents_accessible != nullptr) && (accessible != nullptr));
 
+  bool allocation_map_entry_found = false;
+
   {  // memory_lock protects access to the NMappedNodes array and fragment user data since these may
      // change with calls to memory APIs.
     ScopedAcquire<KernelSharedMutex> lock(&memory_lock_);
@@ -787,8 +837,11 @@ hsa_status_t Runtime::PtrInfo(const void* ptr, hsa_amd_pointer_info_t* info, voi
     // We don't care if this returns an error code.
     // The type will be HSA_EXT_POINTER_TYPE_UNKNOWN if so.
     auto err = hsaKmtQueryPointerInfo(ptr, &thunkInfo);
-    assert(((err == HSAKMT_STATUS_SUCCESS) || (thunkInfo.Type == HSA_POINTER_UNKNOWN)) &&
-           "Thunk ptr info error and not type HSA_POINTER_UNKNOWN.");
+    if (err != HSAKMT_STATUS_SUCCESS || thunkInfo.Type == HSA_POINTER_UNKNOWN) {
+      retInfo.type = HSA_EXT_POINTER_TYPE_UNKNOWN;
+      memcpy(info, &retInfo, retInfo.size);
+      return HSA_STATUS_SUCCESS;
+    }
 
     if (returnListData) {
       assert(thunkInfo.NMappedNodes <= agents_by_node_.size() &&
@@ -816,32 +869,46 @@ hsa_status_t Runtime::PtrInfo(const void* ptr, hsa_amd_pointer_info_t* info, voi
       assert((retInfo.hostBaseAddress || retInfo.agentBaseAddress) && "Thunk pointer info returned no base address.");
       block_info->base = (retInfo.hostBaseAddress ? retInfo.hostBaseAddress : retInfo.agentBaseAddress);
       block_info->length = retInfo.sizeInBytes;
+
+      // Report the owning agent, even if such an agent is not usable in the process.
+      auto nodeAgents = agents_by_node_.find(thunkInfo.Node);
+      assert(nodeAgents != agents_by_node_.end() && "Node id not found!");
+      block_info->agentOwner = nodeAgents->second[0];
     }
     auto fragment = allocation_map_.upper_bound(ptr);
     if (fragment != allocation_map_.begin()) {
       fragment--;
       if ((fragment->first <= ptr) &&
-        (ptr < reinterpret_cast<const uint8_t*>(fragment->first) + fragment->second.size)) {
-          // agent and host address must match here.  Only lock memory is allowed to have differing
-          // addresses but lock memory has type HSA_EXT_POINTER_TYPE_LOCKED and cannot be
-          // suballocated.
-          retInfo.agentBaseAddress = const_cast<void*>(fragment->first);
-          retInfo.hostBaseAddress = retInfo.agentBaseAddress;
-          retInfo.sizeInBytes = fragment->second.size;
-          retInfo.userData = fragment->second.user_ptr;
+          (ptr < reinterpret_cast<const uint8_t*>(fragment->first) + fragment->second.size_requested)) {
+        // agent and host address must match here. Only lock memory is allowed to have differing
+        // addresses but lock memory has type HSA_EXT_POINTER_TYPE_LOCKED and cannot be
+        // suballocated.
+        retInfo.agentBaseAddress = const_cast<void*>(fragment->first);
+        retInfo.hostBaseAddress = retInfo.agentBaseAddress;
+        retInfo.sizeInBytes = fragment->second.size_requested;
+        retInfo.userData = fragment->second.user_ptr;
+        allocation_map_entry_found = true;
       }
     }
   }  // end lock scope
 
-  retInfo.size = Min(size_t(info->size), sizeof(hsa_amd_pointer_info_t));
+  // Return type UNKNOWN for released fragments.  Do not report the underlying block info to users!
+  if ((!allocation_map_entry_found) &&
+      ((retInfo.type == HSA_EXT_POINTER_TYPE_HSA) || (retInfo.type == HSA_EXT_POINTER_TYPE_IPC))) {
+    retInfo.type = HSA_EXT_POINTER_TYPE_UNKNOWN;
+  }
 
   // IPC and Graphics memory may come from a node that does not have an agent in this process.
   // Ex. ROCR_VISIBLE_DEVICES or peer GPU is not supported by ROCm.
+  retInfo.agentOwner.handle = 0;
   auto nodeAgents = agents_by_node_.find(thunkInfo.Node);
-  if (nodeAgents != agents_by_node_.end())
-    retInfo.agentOwner = nodeAgents->second[0]->public_handle();
-  else
-    retInfo.agentOwner.handle = 0;
+  assert(nodeAgents != agents_by_node_.end() && "Node id not found!");
+  for (auto agent : nodeAgents->second) {
+    if (agent->Enabled()) {
+      retInfo.agentOwner = agent->public_handle();
+      break;
+    }
+  }
 
   // Correct agentOwner for locked memory.  Thunk reports the GPU that owns the
   // alias but users are expecting to see a CPU when the memory is system.
@@ -898,6 +965,9 @@ hsa_status_t Runtime::SetPtrInfoData(const void* ptr, void* userptr) {
 hsa_status_t Runtime::IPCCreate(void* ptr, size_t len, hsa_amd_ipc_memory_t* handle) {
   static_assert(sizeof(hsa_amd_ipc_memory_t) == sizeof(HsaSharedMemoryHandle),
                 "Thunk IPC mismatch.");
+
+  static const size_t pageSize = 4096;
+
   // Reject sharing allocations larger than ~8TB due to thunk limitations.
   if (len > 0x7FFFFFFF000ull) return HSA_STATUS_ERROR_INVALID_ARGUMENT;
 
@@ -907,8 +977,14 @@ hsa_status_t Runtime::IPCCreate(void* ptr, size_t len, hsa_amd_ipc_memory_t* han
   info.size = sizeof(info);
   if (PtrInfo(ptr, &info, nullptr, nullptr, nullptr, &block) != HSA_STATUS_SUCCESS)
     return HSA_STATUS_ERROR_INVALID_ARGUMENT;
-  if ((info.agentBaseAddress != ptr) || (info.sizeInBytes != len))
+
+  // Temporary: Previous versions of HIP will call hsa_amd_ipc_memory_create with the len aligned to
+  // granularity. We need to maintain backward compatibility for 2 releases so we temporarily allow
+  // this. After 2 releases, we will only allow info.sizeInBytes != len.
+  if ((info.agentBaseAddress != ptr) ||
+      (info.sizeInBytes != len && AlignUp(info.sizeInBytes, pageSize) != len)) {
     return HSA_STATUS_ERROR_INVALID_ARGUMENT;
+  }
   if ((block.base != ptr) || (block.length != len)) {
     if (!IsMultipleOf(block.base, 2 * 1024 * 1024)) {
       assert(false && "Fragment's block not aligned to 2MB!");
@@ -949,11 +1025,13 @@ hsa_status_t Runtime::IPCAttach(const hsa_amd_ipc_memory_t* handle, size_t len,
   uint32_t fragOffset = 0;
 
   auto fixFragment = [&]() {
-    if (!isFragment) return;
-    importAddress = reinterpret_cast<uint8_t*>(importAddress) + fragOffset;
-    len = Min(len, importSize - fragOffset);
+    if (isFragment) {
+      importAddress = reinterpret_cast<uint8_t*>(importAddress) + fragOffset;
+      len = Min(len, importSize - fragOffset);
+    }
     ScopedAcquire<KernelSharedMutex> lock(&memory_lock_);
-    allocation_map_[importAddress] = AllocationRegion(nullptr, len);
+    allocation_map_[importAddress] =
+        AllocationRegion(nullptr, len, len, core::MemoryRegion::AllocateNoFlags);
   };
 
   if ((importHandle.handle[6] & 0x80000000) != 0) {
@@ -1339,9 +1417,20 @@ Runtime::Runtime()
       kfd_version{0} {}
 
 hsa_status_t Runtime::Load() {
-  flag_.Refresh();
+  os::cpuid_t cpuinfo;
 
+  // Assume features are not supported if parse CPUID fails
+  if (!os::ParseCpuID(&cpuinfo)) {
+    /*
+     * This is not a failure, in some environments such as SRIOV, not all CPUID info is
+     * exposed inside the guest
+     */
+    debug_warning("Parsing CPUID failed.");
+  }
+
+  flag_.Refresh();
   g_use_interrupt_wait = flag_.enable_interrupt();
+  g_use_mwaitx = flag_.check_mwaitx(cpuinfo.mwaitx);
 
   if (!AMD::Load()) {
     return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
@@ -1349,10 +1438,8 @@ hsa_status_t Runtime::Load() {
 
   // Setup system clock frequency for the first time.
   if (sys_clock_freq_ == 0) {
-    // Cache system clock frequency
-    HsaClockCounters clocks;
-    hsaKmtGetClockCounters(0, &clocks);
-    sys_clock_freq_ = clocks.SystemClockFrequencyHz;
+    sys_clock_freq_ = os::SystemClockFrequency();
+    if (sys_clock_freq_ < 100000) debug_warning("System clock resolution is low.");
   }
 
   BindVmFaultHandler();
@@ -1375,10 +1462,15 @@ hsa_status_t Runtime::Load() {
   // Load tools libraries
   LoadTools();
 
+  // Load svm profiler
+  svm_profile_.reset(new AMD::SvmProfileControl);
+
   return HSA_STATUS_SUCCESS;
 }
 
 void Runtime::Unload() {
+  svm_profile_.reset(nullptr);
+
   UnloadTools();
   UnloadExtensions();
 
@@ -1388,6 +1480,9 @@ void Runtime::Unload() {
   std::for_each(gpu_agents_.begin(), gpu_agents_.end(), DeleteObject());
   gpu_agents_.clear();
 
+  std::for_each(disabled_gpu_agents_.begin(), disabled_gpu_agents_.end(), DeleteObject());
+  disabled_gpu_agents_.clear();
+
   async_events_control_.Shutdown();
 
   if (vm_fault_signal_ != nullptr) {
@@ -1489,53 +1584,118 @@ void Runtime::LoadTools() {
   typedef Agent* (*tool_wrap_t)(Agent*);
   typedef void (*tool_add_t)(Runtime*);
 
-  // Load tool libs
+  std::vector<const char*> failed;
+
+  //Get loaded libs and filter to tool libraries.
+  struct lib_t {
+    lib_t(os::LibHandle lib, uint32_t order, std::string name) : lib_(lib), order_(order), name_(name) {}
+    os::LibHandle lib_;
+    uint32_t order_;
+    std::string name_;
+  };
+
+  std::list<lib_t> sorted;
+  uint32_t env_count=0;
+
+  // Load env var tool lib names and determine ordering offset.
   std::string tool_names = flag_.tools_lib_names();
+  std::vector<std::string> names;
   if (tool_names != "") {
-    std::vector<std::string> names = parse_tool_names(tool_names);
-    std::vector<const char*> failed;
-    for (auto& name : names) {
-      os::LibHandle tool = os::LoadLib(name);
+    names = parse_tool_names(tool_names);
+    env_count = names.size();
+  }
 
-      if (tool != NULL) {
-        tool_libs_.push_back(tool);
+  // Discover loaded tools.
+  std::vector<os::LibHandle> loaded = os::GetLoadedToolsLib();
+  for(auto& handle : loaded) {
+    const uint32_t* order = (const uint32_t*)os::GetExportAddress(handle, "HSA_AMD_TOOL_PRIORITY");
+    if(order) {
+      sorted.push_back(lib_t(handle, *order+env_count, os::GetLibraryName(handle)));
+    } else {
+      os::CloseLib(handle);
+    }
+  }
+  
+  // Load env var tools.
+  env_count=0;
+  for (auto& name : names) {
+    os::LibHandle tool = os::LoadLib(name);
 
-        rocr::AMD::callback_t<tool_init_t> ld = (tool_init_t)os::GetExportAddress(tool, "OnLoad");
-        if (ld) {
-          if (!ld(&hsa_api_table_.hsa_api,
-                  hsa_api_table_.hsa_api.version.major_id,
-                  failed.size(), &failed[0])) {
-            failed.push_back(name.c_str());
-            os::CloseLib(tool);
-            continue;
-          }
-        }
+    if (tool != nullptr) {
+      sorted.push_back(lib_t(tool, env_count, name));
+      env_count++;
+    } else {
+      failed.push_back(name.c_str());
+      if (flag().report_tool_load_failures())
+        fprintf(stderr, "Tool lib \"%s\" failed to load.\n", name.c_str());
+    }
+  }
 
-        rocr::AMD::callback_t<tool_wrap_t> wrap =
-            (tool_wrap_t)os::GetExportAddress(tool, "WrapAgent");
-        if (wrap) {
-          std::vector<core::Agent*>* agent_lists[2] = {&cpu_agents_,
-                                                       &gpu_agents_};
-          for (std::vector<core::Agent*>* agent_list : agent_lists) {
-            for (size_t agent_idx = 0; agent_idx < agent_list->size();
-                 ++agent_idx) {
+  if(!sorted.empty()) {
+    // Close duplicate handles
+    sorted.sort([](const lib_t& lhs, const lib_t& rhs) {
+      if(lhs.lib_ == rhs.lib_)
+        return lhs.order_ < rhs.order_;
+      return lhs.lib_ < rhs.lib_;
+    });
+
+    os::LibHandle current = sorted.front().lib_;
+    auto it = sorted.begin();
+    it++;
+    while(it != sorted.end()) {
+      if(it->lib_==current) {
+        os::CloseLib(current);
+        auto rem = it;
+        it = sorted.erase(rem);
+      } else {
+        current = it->lib_;
+        it++;
+      }
+    }
+
+    // Sort to load order
+    sorted.sort([](const lib_t& lhs, const lib_t& rhs) {
+      return lhs.order_ < rhs.order_;
+    });
+    
+    for(auto& lib : sorted) {
+      auto& tool = lib.lib_;
+
+      rocr::AMD::callback_t<tool_init_t> ld = (tool_init_t)os::GetExportAddress(tool, "OnLoad");
+      if (!ld) {
+        failed.push_back(lib.name_.c_str());
+        os::CloseLib(tool);
+        continue;
+      }
+      if (!ld(&hsa_api_table_.hsa_api,
+        hsa_api_table_.hsa_api.version.major_id,
+        failed.size(), failed.data())) {
+          failed.push_back(lib.name_.c_str());
+          os::CloseLib(tool);
+          continue;
+      }
+      tool_libs_.push_back(tool);
+
+      rocr::AMD::callback_t<tool_wrap_t> wrap =
+        (tool_wrap_t)os::GetExportAddress(tool, "WrapAgent");
+      if (wrap) {
+        std::vector<core::Agent*>* agent_lists[2] = {&cpu_agents_,
+          &gpu_agents_};
+        for (std::vector<core::Agent*>* agent_list : agent_lists) {
+          for (size_t agent_idx = 0; agent_idx < agent_list->size();
+            ++agent_idx) {
               Agent* agent = wrap(agent_list->at(agent_idx));
               if (agent != NULL) {
                 assert(agent->IsValid() &&
-                       "Agent returned from WrapAgent is not valid");
+                  "Agent returned from WrapAgent is not valid");
                 agent_list->at(agent_idx) = agent;
               }
-            }
           }
         }
+      }
 
-        rocr::AMD::callback_t<tool_add_t> add = (tool_add_t)os::GetExportAddress(tool, "AddAgent");
-        if (add) add(this);
-      }
-      else {
-        if (flag().report_tool_load_failures())
-          fprintf(stderr, "Tool lib \"%s\" failed to load.\n", name.c_str());
-      }
+      rocr::AMD::callback_t<tool_add_t> add = (tool_add_t)os::GetExportAddress(tool, "AddAgent");
+      if (add) add(this);
     }
   }
 }
@@ -1748,6 +1908,14 @@ hsa_status_t Runtime::SetSvmAttrib(void* ptr, size_t size,
           clear_flags |= HSA_SVM_FLAG_GPU_READ_MOSTLY;
         break;
       }
+      case HSA_AMD_SVM_ATTRIB_GPU_EXEC: {
+        Check(attrib);
+        if (value)
+          set_flags |= HSA_SVM_FLAG_GPU_EXEC;
+        else
+          clear_flags |= HSA_SVM_FLAG_GPU_EXEC;
+        break;
+      }
       case HSA_AMD_SVM_ATTRIB_AGENT_ACCESSIBLE: {
         Agent* agent = Convert(value);
         ConfirmNew(agent);
@@ -2102,7 +2270,6 @@ hsa_status_t Runtime::SvmPrefetch(void* ptr, size_t size, hsa_agent_t agent,
 Agent* Runtime::GetSVMPrefetchAgent(void* ptr, size_t size) {
   uintptr_t base = reinterpret_cast<uintptr_t>(AlignDown(ptr, 4096));
   uintptr_t end = AlignUp(reinterpret_cast<uintptr_t>(ptr) + size, 4096);
-  size_t len = end - base;
 
   std::vector<std::pair<uintptr_t, uintptr_t>> holes;
 
@@ -2155,5 +2322,52 @@ Agent* Runtime::GetSVMPrefetchAgent(void* ptr, size_t size) {
   return agents_by_node_[prefetch_node][0];
 }
 
+hsa_status_t Runtime::DmaBufExport(const void* ptr, size_t size, int* dmabuf, uint64_t* offset) {
+#ifdef __linux__
+  ScopedAcquire<KernelSharedMutex::Shared> lock(memory_lock_.shared());
+  // Lookup containing allocation.
+  auto mem = allocation_map_.upper_bound(ptr);
+  if (mem != allocation_map_.begin()) {
+    mem--;
+    if ((mem->first <= ptr) &&
+        (ptr < reinterpret_cast<const uint8_t*>(mem->first) + mem->second.size)) {
+      // Check size is in bounds.
+      if (uintptr_t(ptr) - uintptr_t(mem->first) + size <= mem->second.size) {
+        // Check allocation is on GPU
+        if (mem->second.region->owner()->device_type() != Agent::kAmdGpuDevice)
+          return HSA_STATUS_ERROR_INVALID_AGENT;
+
+        int fd;
+        uint64_t off;
+        HSAKMT_STATUS err = hsaKmtExportDMABufHandle(const_cast<void*>(ptr), size, &fd, &off);
+        if (err == HSAKMT_STATUS_SUCCESS) {
+          *dmabuf = fd;
+          *offset = off;
+          return HSA_STATUS_SUCCESS;
+        }
+
+        assert((err != HSAKMT_STATUS_INVALID_PARAMETER) &&
+               "Thunk does not recognize an expected allocation.");
+        if (err == HSAKMT_STATUS_ERROR) return HSA_STATUS_ERROR_OUT_OF_RESOURCES;
+        return HSA_STATUS_ERROR;
+      }
+    }
+  }
+  return HSA_STATUS_ERROR_INVALID_ALLOCATION;
+#else
+  return HSA_STATUS_ERROR_NOT_INITIALIZED;
+#endif
+}
+
+hsa_status_t Runtime::DmaBufClose(int dmabuf) {
+#ifdef __linux__
+  int err = close(dmabuf);
+  if (err == 0) return HSA_STATUS_SUCCESS;
+  return HSA_STATUS_ERROR_RESOURCE_FREE;
+#else
+  return HSA_STATUS_ERROR_NOT_INITIALIZED;
+#endif
+}
+
 }  // namespace core
 }  // namespace rocr
