@@ -1,28 +1,28 @@
 /*
- * Copyright © 2007-2019 Advanced Micro Devices, Inc.
- * All Rights Reserved.
- *
- * Permission is hereby granted, free of charge, to any person obtaining
- * a copy of this software and associated documentation files (the
- * "Software"), to deal in the Software without restriction, including
- * without limitation the rights to use, copy, modify, merge, publish,
- * distribute, sub license, and/or sell copies of the Software, and to
- * permit persons to whom the Software is furnished to do so, subject to
- * the following conditions:
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS, AUTHORS
- * AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
- * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
- * The above copyright notice and this permission notice (including the
- * next paragraph) shall be included in all copies or substantial portions
- * of the Software.
- */
+************************************************************************************************************************
+*
+*  Copyright (C) 2007-2022 Advanced Micro Devices, Inc.  All rights reserved.
+*
+* Permission is hereby granted, free of charge, to any person obtaining a
+* copy of this software and associated documentation files (the "Software"),
+* to deal in the Software without restriction, including without limitation
+* the rights to use, copy, modify, merge, publish, distribute, sublicense,
+* and/or sell copies of the Software, and to permit persons to whom the
+* Software is furnished to do so, subject to the following conditions:
+*
+* The above copyright notice and this permission notice shall be included in
+* all copies or substantial portions of the Software.
+*
+* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+* THE COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
+* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+* OTHER DEALINGS IN THE SOFTWARE
+*
+***********************************************************************************************************************/
+
 
 /**
 ************************************************************************************************************************
@@ -37,40 +37,44 @@
 #include "addrlib.h"
 
 namespace rocr {
-namespace Addr
-{
-namespace V2
-{
+namespace Addr {
+namespace V2 {
 
 /**
 ************************************************************************************************************************
 * @brief Flags for SwizzleModeTable
 ************************************************************************************************************************
 */
-struct SwizzleModeFlags
+union SwizzleModeFlags
 {
-    // Swizzle mode
-    UINT_32 isLinear        : 1;    // Linear
+    struct
+    {
+        // Swizzle mode
+        UINT_32 isLinear        : 1;    // Linear
 
-    // Block size
-    UINT_32 is256b          : 1;    // Block size is 256B
-    UINT_32 is4kb           : 1;    // Block size is 4KB
-    UINT_32 is64kb          : 1;    // Block size is 64KB
-    UINT_32 isVar           : 1;    // Block size is variable
+        // Block size
+        UINT_32 is256b          : 1;    // Block size is 256B
+        UINT_32 is4kb           : 1;    // Block size is 4KB
+        UINT_32 is64kb          : 1;    // Block size is 64KB
+        UINT_32 isVar           : 1;    // Block size is variable
 
-    UINT_32 isZ             : 1;    // Z order swizzle mode
-    UINT_32 isStd           : 1;    // Standard swizzle mode
-    UINT_32 isDisp          : 1;    // Display swizzle mode
-    UINT_32 isRot           : 1;    // Rotate swizzle mode
+        UINT_32 isZ             : 1;    // Z order swizzle mode
+        UINT_32 isStd           : 1;    // Standard swizzle mode
+        UINT_32 isDisp          : 1;    // Display swizzle mode
+        UINT_32 isRot           : 1;    // Rotate swizzle mode
 
-    // XOR mode
-    UINT_32 isXor           : 1;    // XOR after swizzle if set
+        // XOR mode
+        UINT_32 isXor           : 1;    // XOR after swizzle if set
 
-    UINT_32 isT             : 1;    // T mode
+        UINT_32 isT             : 1;    // T mode
 
-    UINT_32 isRtOpt         : 1;    // mode opt for render target
+        // GFX10
+        UINT_32 isRtOpt         : 1;    // mode opt for render target
 
-    UINT_32 reserved        : 20;   // Reserved bits
+        UINT_32 reserved        : 20;   // Reserved bits
+    };
+
+    UINT_32 u32All;
 };
 
 struct Dim2d
@@ -89,15 +93,18 @@ struct Dim3d
 // Macro define resource block type
 enum AddrBlockType
 {
-    AddrBlockMicro     = 0, // Resource uses 256B block
-    AddrBlockThin4KB   = 1, // Resource uses thin 4KB block
-    AddrBlockThick4KB  = 2, // Resource uses thick 4KB block
-    AddrBlockThin64KB  = 3, // Resource uses thin 64KB block
-    AddrBlockThick64KB = 4, // Resource uses thick 64KB block
-    AddrBlockVar       = 5, // Resource uses var block, only valid for GFX9
-    AddrBlockLinear    = 6, // Resource uses linear swizzle mode
+    AddrBlockLinear    = 0, // Resource uses linear swizzle mode
+    AddrBlockMicro     = 1, // Resource uses 256B block
+    AddrBlockThin4KB   = 2, // Resource uses thin 4KB block
+    AddrBlockThick4KB  = 3, // Resource uses thick 4KB block
+    AddrBlockThin64KB  = 4, // Resource uses thin 64KB block
+    AddrBlockThick64KB = 5, // Resource uses thick 64KB block
+    AddrBlockThinVar   = 6, // Resource uses thin var block
+    AddrBlockThickVar  = 7, // Resource uses thick var block
+    AddrBlockMaxTiledType,
 
-    AddrBlockMaxTiledType = AddrBlockVar + 1,
+    AddrBlockThin256KB  = AddrBlockThinVar,
+    AddrBlockThick256KB = AddrBlockThickVar,
 };
 
 enum AddrSwSet
@@ -118,6 +125,87 @@ const UINT_32 Log2Size256 = 8u;
 const UINT_32 Log2Size4K  = 12u;
 const UINT_32 Log2Size64K = 16u;
 
+/**
+************************************************************************************************************************
+* @brief Bit setting for swizzle pattern
+************************************************************************************************************************
+*/
+union ADDR_BIT_SETTING
+{
+    struct
+    {
+        UINT_16 x;
+        UINT_16 y;
+        UINT_16 z;
+        UINT_16 s;
+    };
+    UINT_64 value;
+};
+
+/**
+************************************************************************************************************************
+* @brief Swizzle pattern information
+************************************************************************************************************************
+*/
+struct ADDR_SW_PATINFO
+{
+    UINT_8  maxItemCount;
+    UINT_8  nibble01Idx;
+    UINT_16 nibble2Idx;
+    UINT_16 nibble3Idx;
+    UINT_8  nibble4Idx;
+};
+
+/**
+************************************************************************************************************************
+*   InitBit
+*
+*   @brief
+*       Initialize bit setting value via a return value
+************************************************************************************************************************
+*/
+#define InitBit(c, index) (1ull << ((c << 4) + index))
+
+const UINT_64 X0  = InitBit(0,  0);
+const UINT_64 X1  = InitBit(0,  1);
+const UINT_64 X2  = InitBit(0,  2);
+const UINT_64 X3  = InitBit(0,  3);
+const UINT_64 X4  = InitBit(0,  4);
+const UINT_64 X5  = InitBit(0,  5);
+const UINT_64 X6  = InitBit(0,  6);
+const UINT_64 X7  = InitBit(0,  7);
+const UINT_64 X8  = InitBit(0,  8);
+const UINT_64 X9  = InitBit(0,  9);
+const UINT_64 X10 = InitBit(0, 10);
+const UINT_64 X11 = InitBit(0, 11);
+
+const UINT_64 Y0  = InitBit(1,  0);
+const UINT_64 Y1  = InitBit(1,  1);
+const UINT_64 Y2  = InitBit(1,  2);
+const UINT_64 Y3  = InitBit(1,  3);
+const UINT_64 Y4  = InitBit(1,  4);
+const UINT_64 Y5  = InitBit(1,  5);
+const UINT_64 Y6  = InitBit(1,  6);
+const UINT_64 Y7  = InitBit(1,  7);
+const UINT_64 Y8  = InitBit(1,  8);
+const UINT_64 Y9  = InitBit(1,  9);
+const UINT_64 Y10 = InitBit(1, 10);
+const UINT_64 Y11 = InitBit(1, 11);
+
+const UINT_64 Z0  = InitBit(2,  0);
+const UINT_64 Z1  = InitBit(2,  1);
+const UINT_64 Z2  = InitBit(2,  2);
+const UINT_64 Z3  = InitBit(2,  3);
+const UINT_64 Z4  = InitBit(2,  4);
+const UINT_64 Z5  = InitBit(2,  5);
+const UINT_64 Z6  = InitBit(2,  6);
+const UINT_64 Z7  = InitBit(2,  7);
+const UINT_64 Z8  = InitBit(2,  8);
+
+const UINT_64 S0  = InitBit(3,  0);
+const UINT_64 S1  = InitBit(3,  1);
+const UINT_64 S2  = InitBit(3,  2);
+
 /**
 ************************************************************************************************************************
 * @brief This class contains asic independent address lib functionalities
@@ -209,6 +297,10 @@ public:
         const ADDR2_COMPUTE_SUBRESOURCE_OFFSET_FORSWIZZLEPATTERN_INPUT* pIn,
         ADDR2_COMPUTE_SUBRESOURCE_OFFSET_FORSWIZZLEPATTERN_OUTPUT*      pOut);
 
+    ADDR_E_RETURNCODE ComputeNonBlockCompressedView(
+        const ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_INPUT* pIn,
+        ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_OUTPUT*      pOut);
+
     ADDR_E_RETURNCODE Addr2GetPreferredSurfaceSetting(
         const ADDR2_GET_PREFERRED_SURF_SETTING_INPUT* pIn,
         ADDR2_GET_PREFERRED_SURF_SETTING_OUTPUT*      pOut) const;
@@ -237,11 +329,7 @@ protected:
 
     BOOL_32 IsValidSwMode(AddrSwizzleMode swizzleMode) const
     {
-        // Don't dereference a reinterpret_cast pointer so as not to break
-        // strict-aliasing rules.
-        UINT_32 mode;
-        memcpy(&mode, &m_swizzleModeTable[swizzleMode], sizeof(UINT_32));
-        return mode != 0;
+        return (m_swizzleModeTable[swizzleMode].u32All != 0);
     }
 
     // Checking block size
@@ -458,12 +546,18 @@ protected:
         return ADDR_NOTSUPPORTED;
     }
 
-    virtual ADDR_E_RETURNCODE HwlComputeDccAddrFromCoord(
+    virtual ADDR_E_RETURNCODE HwlSupportComputeDccAddrFromCoord(
+        const ADDR2_COMPUTE_DCC_ADDRFROMCOORD_INPUT* pIn)
+    {
+        ADDR_NOT_IMPLEMENTED();
+        return ADDR_NOTSUPPORTED;
+    }
+
+    virtual VOID HwlComputeDccAddrFromCoord(
         const ADDR2_COMPUTE_DCC_ADDRFROMCOORD_INPUT* pIn,
         ADDR2_COMPUTE_DCC_ADDRFROMCOORD_OUTPUT*      pOut)
     {
         ADDR_NOT_IMPLEMENTED();
-        return ADDR_NOTSUPPORTED;
     }
 
     virtual ADDR_E_RETURNCODE HwlComputeCmaskAddrFromCoord(
@@ -559,6 +653,14 @@ protected:
         return ADDR_NOTSUPPORTED;
     }
 
+    virtual ADDR_E_RETURNCODE HwlComputeNonBlockCompressedView(
+        const ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_INPUT* pIn,
+        ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_OUTPUT*      pOut) const
+    {
+        ADDR_NOT_IMPLEMENTED();
+        return ADDR_NOTSUPPORTED;
+    }
+
     virtual ADDR_E_RETURNCODE HwlGetPreferredSurfaceSetting(
         const ADDR2_GET_PREFERRED_SURF_SETTING_INPUT* pIn,
         ADDR2_GET_PREFERRED_SURF_SETTING_OUTPUT*      pOut) const
@@ -822,6 +924,22 @@ protected:
         AddrResourceType  resourceType,
         UINT_32           elemLog2) const;
 
+    static BOOL_32 IsBlockTypeAvaiable(ADDR2_BLOCK_SET blockSet, AddrBlockType blockType);
+
+    static BOOL_32 BlockTypeWithinMemoryBudget(
+        UINT_64 minSize,
+        UINT_64 newBlockTypeSize,
+        UINT_32 ratioLow,
+        UINT_32 ratioHi,
+        DOUBLE  memoryBudget = 0.0f,
+        BOOL_32 newBlockTypeBigger = TRUE);
+
+#if DEBUG
+    VOID ValidateStereoInfo(
+        const ADDR2_COMPUTE_SURFACE_INFO_INPUT* pIn,
+        const ADDR2_COMPUTE_SURFACE_INFO_OUTPUT* pOut) const;
+#endif
+
     UINT_32 m_se;                       ///< Number of shader engine
     UINT_32 m_rbPerSe;                  ///< Number of render backend per shader engine
     UINT_32 m_maxCompFrag;              ///< Number of max compressed fragment
@@ -866,6 +984,5 @@ private:
 } // Addr
 } // rocr
 
-
 #endif
 
